<?php

/**
 * The Microsite class
 */
class Microsite {

  /**
   * Internal ID
   */
  public $id;

  /**
   * Microsite key
   */
  public $name;

  /**
   * Node ID
   */
  public $nid;

  /**
   * The node type this microsite is attached to
   */
  public $node_type;

  /**
   * Status
   */
  public $status;

  /**
   * Any other settings
   */
  public $data = array();

  /**
   * The related Node
   */
  public $node;

  /**
   * Constructor
   *
   * @param array $config
   *  A microsite configuration array. The following keys may be set:
   *    'id'  The ID for the microsite (required)
   *    'nid' The related node ID (optional)
   *    'title  A title for the microsite (optional)
   *    'mlid'  The generated menu ID (optional)
   *    'data'  An array of other data properties to store (optional)
   *
   * @return \Microsite
   * Either the created Microsite, or FALSE if incorrect data was provided
   */
  public function __construct($config = array()) {
    return $this->createFromConfig($config);
  }

  /**
   * Load config values into this object
   *
   * @param array $config
   *  A microsite configuration array. The following keys may be set:
   *    'id'  The ID for the microsite (required)
   *    'nid' The related node ID (optional)
   *    'title  A title for the microsite (optional)
   *    'mlid'  The generated menu ID (optional)
   *    'data'  An array of other data properties to store (optional)
   *
   * @return \Microsite
   * Either the created Microsite, or FALSE if incorrect data was provided
   */
  public function createFromConfig($config) {

    if (!empty($config)) {

      // Data may come in several shapes
      if (isset($config['data']) && !is_array($config['data'])) {
        $this->data = unserialize($config['data']);
      }
      elseif (isset($config['data']) && is_array($config['data'])) {
        $this->data = $config['data'];
      }
      else {
        $this->data = array();
      }

      // If we have a node, we can load some configuration off that
      if (isset($config['node'])) {
        $this->node = $config['node'];
        $this->nid = $this->node->nid;
        $this->name = microsites_make_key($this->node);
        $this->node_type = $this->node->type;

        return $this;
      }
      else {
        isset($config['id']) ? $this->id = $config['id'] : $this->id = '';
        isset($config['name']) ? $this->name = $config['name'] : $this->name = '';
        isset($config['node_type']) ? $this->node_type = $config['node_type'] : $this->node_type = '';
        if (isset($config['nid'])) {
          $this->nodeLoad($config['nid']);
        }
        if(isset($config['status'])) {
          $this->status = $config['status'];
        }

        return $this;
      }
    }
    return FALSE;
  }

  /**
   * Helper to ensure the Node is loaded
   *
   * @param $nid
   *
   * @return \Microsite
   */
  public function nodeLoad($nid) {
    $this->nid = $nid;
    if($node = node_load($this->nid)) {
      $this->node = $node;
      $this->node_type = $node->type;
    }
    return $this;
  }

  /**
   * Enable this microsite
   */
  public function enable() {
    $this->status = MICROSITES_ENABLED;

    // Let plugins perform enable actions
    $this->plugins('enable');
  }

  /**
   * Reset and enable the microsite
   *
   * Reverts the microsite to its default configuration and enables it
   */
  public function reset() {
    // Give plugins a chance to clean up
    $this->plugins('reset');

    // Erase the data array
    $this->data = array();
  }

  /**
   * Run on initialisation of a live microsite
   */
  public function active() {
    $this->plugins('active');
  }

  /**
   * Enable this microsite
   */
  public function disable() {
    $this->status = MICROSITES_DISABLED;

    // Let plugins perform disable actions
    $this->plugins('disable');
  }

  /**
   * Delete this microsite
   *
   * This runs just prior to deletion
   */
  public function delete() {
    // Give plugins a chance to clean up
    $this->plugins('delete');
  }

  /**
   * Generate a configuration form based on plugins
   *
   * @param $form
   *  The Configuration form
   */
  public function configForm(&$form) {
    $args = array('form' => &$form);
    $this->plugins('configForm', $args);
  }

  /**
   * Save the form submit results from a config form
   *
   * @param $form
   * @param $form_state
   */
  public function configFormSave(&$form, $form_state) {
    $args = array('form' => &$form, 'form_state' => $form_state);
    $this->plugins('configFormSave', $args);
  }

  /**
   * Iterate over plugins and execute a callback
   *
   * @param $callback
   *  The callback to use
   * @param array $args
   *  Any arguments required by the callback
   */
  public function plugins($callback, &$args = array()) {
    static $plugins = array();

    if (empty($plugins)) {
      $plugins = microsites_get_extensions();
    }

    foreach ($plugins as $definition) {
      $plugin = $this->pluginInit($definition);
      if (method_exists($plugin, 'requirements') && $plugin->requirements() == TRUE) {
        $this->pluginExecute($plugin, $callback, $args);
      }
    }
  }

  /**
   * Instantiate a plugin
   *
   * @param $def
   *  A plugin definition loaded from ctools
   *
   * @return
   *  An instance of the defined plugin as a Class
   */
  protected function pluginInit($def) {
    $class_name = ctools_plugin_get_class($def, 'class');
    $visitor = new $class_name($def, $this);

    return $visitor;
  }

  /**
   * Call a method on a plugin
   *
   * @param $plugin
   *  An instantiated plugin object
   * @param $callback
   *  The callbac to run
   * @param array $args
   *  Any arguments to pass to the callback
   *
   * @return mixed The result of the callback, if anything
   */
  protected function pluginExecute($plugin, $callback, &$args = array()) {
    if (method_exists($plugin, $callback)) {
      if (!empty($args)) {
        $plugin->{$callback}($args);
      }
      $plugin->{$callback}();
    }
  }

  /**
   * Return the current status
   */
  public function status() {
    if (isset($this->status)) {
      return $this->status;
    }
    return MICROSITES_DISABLED;
  }
}
