<?php
/**
 * @file
 * Administration functions for Microsites
 */

/**
 * Administration page
 *
 * @param $form_state
 *
 * @return array
 */
function microsites_admin_settings_page(&$form_state) {
  $types = node_get_types();
  $settings = microsites_get_types();
  $form = array();

  if (!empty ($types)) {
    $form['description'] = array(
      '#value' => t('Settings for Microsites'),
      '#weight' => '-10',
    );
    $form['enabled_types'] = array(
      '#type' => 'checkboxes',
      '#title' => t('Types to enable'),
    );
    foreach ($types as $type_name => $type) {
      $form['enabled_types']['#options'][$type_name] = $type->name;
    }
    if (!empty($settings)) {
      $form['enabled_types']['#default_value'] = $settings;
    }
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save'),
      '#weight' => '10'
    );
  }
  else {
    $form['no_types'] = array(
      '#value' => t('You have not defined any content types.'),
    );
  }

  return $form;
}

/**
 * Submit handler for admin_settings_page
 *
 * @param $form
 * @param $form_state
 */
function microsites_admin_settings_page_submit(&$form, $form_state) {
  $save_types = array();
  foreach ($form_state['values']['enabled_types'] as $type_name => $type) {
    if (isset($type) && !empty($type)) {
      $save_types[] = $type_name;
    }
  }
  drupal_set_message('Settings saved for microsites');

  microsites_save_types($save_types);
}

/**
 * Page callback for the Microsites listing page
 */
function microsites_admin_listing_page() {
  $output = '';

  $rows = array();
  $header = array('Base node', 'Machine name', 'Status', 'Linked menu', 'Operations');
  $microsites = microsites_config_load_all();

  // Sort microsites into node-type sets and prepare for themeing
  foreach ($microsites as $site) {

    $node = node_load($site['nid']);
    $row = array();
    if ($site['status'] == MICROSITES_ENABLED) {
      $state = t('Enabled');
    }
    else {
      $state = t('Disabled');
    }
    $row[] = array('data' => $node->title);
    $row[] = array('data' => $site['name']);
    $row[] = array('data' => $state);
    $row[] = array('data' => l('Edit', 'admin/build/menu-customize/microsite-' . $site['name']));

    $base_node = l('View base node', 'node/'. $node->nid);
    $edit_ms = l('Edit microsite', 'node/' . $node->nid . '/microsite');
    $row[] = array('data' => $base_node .' | '. $edit_ms);
    $rows[$node->type][] = $row;
  }

  $types = node_get_types();

  // Output microsites by node base type
  foreach ($rows as $type_name => $type) {
    $output .= '<h2>' . check_plain($types[$type_name]->name) .'</h2>';
    $output .= theme('table', $header, $type);

  }

  return $output;
}

/**
 * Callback to list installed plugins
 */
function microsites_admin_plugins() {
  xu('plugin_tools');

  $output = '<h2>Extensions</h2>';

  $output = xu_ctools_plugins_list('microsites', 'extension');

  return $output;
}

/**
 * Per node microsites settings
 *
 * @param $node
 *
 * @return null|string
 */
function microsite_admin_per_node($node) {
  drupal_set_title('Microsite settings');

  $state = MICROSITES_DISABLED;
  $microsite = microsites_load($node);

  if ($microsite->status == MICROSITES_ENABLED) {
    $state = MICROSITES_ENABLED;
  }

  $output = drupal_get_form('microsite_admin_per_node_status', $state);
  if (!empty($microsite)) {
    $output .= drupal_get_form('microsite_admin_per_node_config', $microsite, $state);
    $output .= drupal_get_form('microsite_admin_per_node_data', $microsite);
  }
  return $output;
}

/**
 * Form callback for the per node status form
 *
 * @param $form_state
 * @param $state
 *
 * @return array
 */
function microsite_admin_per_node_status(&$form_state, $state) {
  $form = array();

  xu('theme');
  xu_add_css('#microsite-admin-per-node-status label {float: left;display: inline-block;padding: 1px 10px 0 0;width: 12%;text-align: right;}');

  if ($state == MICROSITES_ENABLED) {
    $state_string = 'Enabled';
  }
  else {
    $state_string = 'Disabled';
  }

  $form['microsite'] = array(
    '#type' => 'fieldset',
    '#title' => t('Status'),
    '#description' => 'This type supports creation of Microsites. Microsites can be disabled after creation without losing content, and can only be deleted if they are disabled.'
  );
  $form['microsite']['enabled'] = array(
    '#type' => 'select',
    '#title' => t('Microsite status'),
    '#options' => array(
      MICROSITES_ENABLED => t('Enabled'),
      MICROSITES_DISABLED => t('Disabled')
    ),
    '#default_value' => $state
  );
  $form['microsite']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => '10'
  );

  if ($state == MICROSITES_DISABLED) {
    $form['clear'] = array(
      '#type' => 'fieldset',
      '#title' => t('Reset/delete'),
      '#description' => 'Resetting clears all configuration and settings for this Microsite but leaves it in the database. Deletion removes it entirely.'
    );
    $form['clear']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset'),
      '#weight' => '8'
    );
    $form['clear']['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#weight' => '8'
    );
  }

  return $form;
}

/**
 * Submit handler for admin_settings_page
 */
function microsite_admin_per_node_status_submit(&$form, $form_state) {
  $node = node_load(arg(1));

  // Load a Microsite from storage, or instantiate one if none exists
  $microsite = microsites_load($node);
  if (!is_object($microsite) && get_class($microsite) != 'Microsite') {
    $microsite = microsites_create($node);
  }

  // Reset
  if($form_state['clicked_button']['#id'] == 'edit-reset') {
    $microsite->reset();
  }
  // Delete
  elseif ($form_state['clicked_button']['#id'] == 'edit-delete') {
    $microsite->delete();
    microsites_delete($microsite->id);
  }
  // Enable or disable
  else {
    // Set the correct status
    if (isset($form_state['values']['enabled']) && $form_state['values']['enabled'] == MICROSITES_ENABLED && $microsite->status() == MICROSITES_DISABLED) {
      drupal_set_message('The microsite has been enabled');
      $microsite->enable();
    }
    elseif($microsite->status() == MICROSITES_ENABLED) {
      drupal_set_message('The microsite has been disabled');
      $microsite->disable();
    }
  }

  // Save
  microsites_save($microsite);
}

/**
 * Form callback for the per node status form
 *
 * @param $form_state
 * @param Microsite $site
 *
 * @param $state
 *
 * @return array
 */
function microsite_admin_per_node_config(&$form_state, $site, $state) {
  $form = array();

  $form['config'] = array(
    '#type' => 'fieldset',
    '#title' => t('Configuration'),
  );

  $site->configForm($form['config']);

  $form['config']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => '10'
  );

  return $form;
}

/**
 * Submit handler for admin_settings_page
 *
 * @param $form
 * @param $form_state
 */
function microsite_admin_per_node_config_submit(&$form, $form_state) {
  $node = node_load(arg(1));

  // Load a Microsite from storage, or instantiate one if none exists
  $microsite = microsites_load($node);
  if (!is_object($microsite) && get_class($microsite) != 'Microsite') {
    // Somethings gone horribly wrong
    drupal_set_message('Microsite has borked this operation. Please try again');
  }

  $microsite->configFormSave($form, $form_state);
  microsites_save($microsite);

  drupal_set_message('Settings saved for microsite');
}

/**
 * Form callback for state data
 */
function microsite_admin_per_node_data(&$form_state, $site) {
  $form = array();

  xu('theme');
  $form_helper = xu_helper('form');
  xu_add_css('#microsite-admin-per-node-data label {float: left;display: inline-block;padding: 3px 10px 0 0;width: 10%;text-align: right;}');

  $form['data'] = array(
    '#type' => 'fieldset',
    '#title' => t('Current state'),
    '#description' => 'This is the current microsite configuration, as stored in the database and including any overrides.'
  );
  $form['data']['style'] = array(
    '#value' => ''
  );
  $form['data']['id'] = array(
    '#type' => 'textfield',
    '#disabled' => TRUE,
    '#title' => t('ID'),
    '#value' => isset($site->id) ? $site->id : '',
  );
  $form_helper->fieldAutosize($form['data']['id']);

  $form['data']['name'] = array(
    '#type' => 'textfield',
    '#disabled' => TRUE,
    '#title' => t('Name'),
    '#value' => isset($site->name) ? $site->name : '',
  );
  $form_helper->fieldAutosize($form['data']['name']);

  $form['data']['nid'] = array(
    '#type' => 'textfield',
    '#disabled' => TRUE,
    '#title' => t('Node ID'),
    '#value' => isset($site->nid) ? $site->nid : '',
  );
  $form_helper->fieldAutosize($form['data']['nid']);

  $form['data']['node_type'] = array(
    '#type' => 'textfield',
    '#disabled' => TRUE,
    '#title' => t('Node type'),
    '#value' => isset($site->node_type) ? $site->node_type : '',
  );
  $form_helper->fieldAutosize($form['data']['node_type']);

  if (!empty($site->data)) {
    foreach ($site->data as $plugin => $settings) {
      if (!empty($settings)) {
        $plugin_settings = microsites_get_extension($plugin);
        $form['data'][$plugin] = array(
          '#type' => 'fieldset',
          '#title' => $plugin_settings['title'],
        );
        foreach ($settings as $key => $value) {
          $length = strlen($value);
          $form['data'][$plugin][$key] = array(
            '#type' => 'textfield',
            '#disabled' => TRUE,
            '#title' => check_plain($key),
            '#value' => check_plain($value),
          );
          $form_helper->fieldAutosize($form['data'][$plugin][$key]);
        }
      }
    }
  }
  return $form;
}

/**
 * Helper to calculate a field length based on content
 */
